package main

import (
	"encoding/csv"
	"fmt"
	"log"
	"math"
	"os"
	"sort"
	"strconv"
)

func mannWhitneyU(x, y []float64) (float64, float64) {
	n1 := len(x)
	n2 := len(y)

	combined := append(x, y...)
	sort.Float64s(combined)

	ranks := make(map[float64]float64)
	for i := 0; i < len(combined); {
		j := i
		for j < len(combined) && combined[i] == combined[j] {
			j++
		}
		rank := float64(i+j+1) / 2
		for k := i; k < j; k++ {
			ranks[combined[k]] = rank
		}
		i = j
	}

	r1 := sumRanks(x, ranks)
	r2 := sumRanks(y, ranks)

	u1 := r1 - float64(n1*(n1+1))/2
	u2 := r2 - float64(n2*(n2+1))/2

	u := math.Min(u1, u2)
	meanU := float64(n1*n2) / 2
	sdU := math.Sqrt(float64(n1*n2*(n1+n2+1)) / 12)

	z := (u - meanU) / sdU
	p := 2 * (1 - normCdf(math.Abs(z)))

	return u, p
}

func sumRanks(arr []float64, ranks map[float64]float64) float64 {
	sum := 0.0
	for _, v := range arr {
		sum += ranks[v]
	}
	return sum
}

func normCdf(x float64) float64 {
	return 0.5 * (1 + math.Erf(x/math.Sqrt2))
}

func main() {
	file, err := os.Open("/media/dell/HIKSEMI/3.2切入切出数据包/cut_2024-04-03-14-06-25/testU.csv")
	if err != nil {
		log.Fatal(err)
	}
	defer file.Close()

	reader := csv.NewReader(file)
	records, err := reader.ReadAll()
	if err != nil {
		log.Fatal(err)
	}

	col1Index := -1
	col2Index := -1
	for i, header := range records[0] {
		if header == "col1" {
			col1Index = i
		} else if header == "col2" {
			col2Index = i
		}
	}

	if col1Index != -1 && col2Index != -1 {
		Slice1 := make([]float64, len(records)-1)
		Slice2 := make([]float64, len(records)-1)
		for i, record := range records[1:] {
			value1, err1 := strconv.ParseFloat(record[col1Index], 64)
			value2, err2 := strconv.ParseFloat(record[col2Index], 64)
			if err1 != nil || err2 != nil {
				log.Fatal("数据转换错误")
			}
			Slice1[i] = value1
			Slice2[i] = value2
		}

		u, p := mannWhitneyU(Slice1, Slice2)
		fmt.Printf("U: %v, P-value: %v\n", u, p)
	} else {
		fmt.Println("未找到名为 'ego_Steering_real' 和 'ego_Steering_cmd' 的列")
	}
}
