package main

import (
	"awesomeProject/entity"
	"awesomeProject/kinglong_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
	"sort"
	"time"
)

var (
	Framenum    int64 = 0
	param       entity.PjisuvParam
	ObjectSlice = make(map[uint32][][]float32)
	DriverSlice = [][]float64{{}, {}, {}}
	AlgSlice    = [][]float64{{}, {}, {}}
	StartTime   int64
	FollowFlag  = false
)

func mannWhitneyU(x, y []float64) (float64, float64) {
	n1 := len(x)
	n2 := len(y)

	combined := append(x, y...)
	sort.Float64s(combined)

	ranks := make(map[float64]float64)
	for i := 0; i < len(combined); {
		j := i
		for j < len(combined) && combined[i] == combined[j] {
			j++
		}
		rank := float64(i+j+1) / 2
		for k := i; k < j; k++ {
			ranks[combined[k]] = rank
		}
		i = j
	}

	r1 := sumRanks(x, ranks)
	r2 := sumRanks(y, ranks)

	u1 := r1 - float64(n1*(n1+1))/2
	u2 := r2 - float64(n2*(n2+1))/2

	u := math.Min(u1, u2)
	meanU := float64(n1*n2) / 2
	sdU := math.Sqrt(float64(n1*n2*(n1+n2+1)) / 12)

	z := (u - meanU) / sdU
	p := 2 * (1 - normCdf(math.Abs(z)))

	return u, p
}

func sumRanks(arr []float64, ranks map[float64]float64) float64 {
	sum := 0.0
	for _, v := range arr {
		sum += ranks[v]
	}
	return sum
}

func normCdf(x float64) float64 {
	return 0.5 * (1 + math.Erf(x/math.Sqrt2))
}

func main() {
	ticker := time.NewTicker(5 * time.Second)
	defer ticker.Stop()
	go listener()
	for {
		select {
		case <-ticker.C:
			FinalCallback()
		}
	}
}

func FinalCallback() {

	if FollowFlag == true {
		ObjectSlice = make(map[uint32][][]float32)
		_, p1 := mannWhitneyU(DriverSlice[0], AlgSlice[0])
		_, p2 := mannWhitneyU(DriverSlice[1], AlgSlice[1])
		_, p3 := mannWhitneyU(DriverSlice[2], AlgSlice[2])
		if p1 < 0.05 || p2 < 0.05 || p3 < 0.05 {
			event_lable := "FollowDifference"
			fmt.Println(event_lable)
			DriverSlice = [][]float64{{}, {}, {}}
			AlgSlice = [][]float64{{}, {}, {}}

		}

	}

}

func CallbackCicvLocation(data *kinglong_msgs.PerceptionLocalization) {
	param.YawOfCicvLocation = data.Yaw
	param.AngularVelocityZOfCicvLocation = data.AngularVelocityZ
}
func CallbackTpperception(data *kinglong_msgs.PerceptionObjects) {
	Framenum += 1
	tempFlag := false
	FollowFlag = false
	for _, obj := range data.Objs {
		if obj.X >= 5 && obj.X <= 50 && math.Abs(float64(obj.Y)) < 1.5 && (obj.Type == 1 || obj.Type == 0) {
			tempFlag = true
		}
	}
	if tempFlag {
		if StartTime == 0 {
			StartTime = time.Now().Unix()
		}
		if time.Now().Unix()-StartTime > 3 {
			FollowFlag = true
		}
	} else {
		StartTime = 0
	}

}

func CallbackDataRead(data *kinglong_msgs.Retrieval) {
	DriverSlice[0] = append(DriverSlice[0], data.StrgAngleRealValue) //方向盘实际转角
	DriverSlice[1] = append(DriverSlice[1], data.VcuAccelPosValue)   //实际加速踏板位置
	DriverSlice[2] = append(DriverSlice[2], data.VcuBrkPelPosValue)  //实际制动踏板位置

}

func CallbackJinlongControlPub(data *kinglong_msgs.JinlongControlCommand) {
	AlgSlice[0] = append(AlgSlice[0], data.ASStrgAngleReq)      //算法输出方向盘转角
	AlgSlice[1] = append(AlgSlice[1], data.ASAutoDAccelPosReq)  //算法输出加速踏板位置
	AlgSlice[2] = append(AlgSlice[2], data.ASAutoDBrkPelPosReq) //算法输出制动踏板位置
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subTpperception, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "tpperception",
		Callback: CallbackTpperception,
	})
	if err != nil {
		panic(err)
	}
	defer subTpperception.Close()

	// create a subscriber
	subControl, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "/jinlong_control_pub",
		Callback: CallbackJinlongControlPub,
	})
	if err != nil {
		panic(err)
	}
	defer subControl.Close()

	// create a subscriber
	subDataRead, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "/data_read",
		Callback: CallbackDataRead,
	})
	if err != nil {
		panic(err)
	}
	defer subDataRead.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
