package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/geometry_msgs"
	"math"
	"os"
	"os/signal"
	"time"
)

type Point struct {
	Latitude  float64
	Longitude float64
}

var (
	param          entity.PjisuvParam
	StartTime      int64
	IsStopped      bool
	IsEndPoint     bool
	IsTrafficLight bool
	count1         int64
	//定义园区４个信号灯的坐标
	point2 = Point{39.72975930689718, 116.48861102824081}
	point3 = Point{39.7288805296616, 116.48812315228867}
	point4 = Point{39.73061430369551, 116.49225103553502}
	point5 = Point{39.73077491578002, 116.49060085035634}

	EndPoint   = Point{0, 0}
	pointlist1 = []Point{point2, point3, point4, point5}
	pointlist2 = []Point{EndPoint}
)

func IfEnter(pointlist []Point, radius float64, lat, lon float64) bool {
	// 判断是否进入点列表中的区域
	point1 := Point{Latitude: lat, Longitude: lon}
	for _, point := range pointlist {
		d := distance(point1, point)
		if d <= radius {
			return true
		}
	}
	return false
}

// 计算两点之间的距离（米）
func distance(point1, point2 Point) float64 {
	// 经纬度转弧度
	lat1 := point1.Latitude * math.Pi / 180
	lon1 := point1.Longitude * math.Pi / 180
	lat2 := point2.Latitude * math.Pi / 180
	lon2 := point2.Longitude * math.Pi / 180

	// 计算距离
	dlon := lon2 - lon1
	dlat := lat2 - lat1
	a := math.Sin(dlat/2)*math.Sin(dlat/2) + math.Sin(dlon/2)*math.Sin(dlon/2)*math.Cos(lat1)*math.Cos(lat2)
	c := 2 * math.Atan2(math.Sqrt(a), math.Sqrt(1-a))
	d := 6371000 * c

	return d
}
func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	//print(param.AutomodeOfPjVehicleFdbPub)
	if count1%10 == 0 {
		IsTrafficLight = IfEnter(pointlist1, 30.0, data.Latitude, data.Longitude)
		IsEndPoint = IfEnter(pointlist2, 5.0, data.Latitude, data.Longitude)
		if param.AutomodeOfPjVehicleFdbPub == 1 && !IsTrafficLight && !IsEndPoint {
			if data.VelocityX < 0.5 {
				// 如果之前没有记录开始时间，记录当前时间
				if StartTime == 0 {
					StartTime = time.Now().Unix()
				}

				// 判断是否持续超过一分钟
				if time.Now().Unix()-StartTime > 3 {
					if !IsStopped {
						event_label := "AbnormalParking"
						fmt.Println(event_label)
						IsStopped = true
					}
				}
			} else {
				// 如果速度大于 0.1，重置开始时间和停止标志
				StartTime = 0
				IsStopped = false
			}
		}
		count1 = 1
	} else {
		count1++
	}
}

func CallbackPjVehicleFdbPub(data *pjisuv_msgs.VehicleFdb) {
	param.AutomodeOfPjVehicleFdbPub = data.Automode
}
func CallbackEndPointMessage(data *geometry_msgs.Point) {
	EndPoint.Longitude = data.X
	EndPoint.Latitude = data.Y

}
func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subTpperception, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "pj_vehicle_fdb_pub",
		Callback: CallbackPjVehicleFdbPub,
	})
	if err != nil {
		panic(err)
	}
	defer subTpperception.Close()

	// create a subscriber
	subEndPointMessage, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "/end_point_message",
		Callback: CallbackEndPointMessage,
	})
	if err != nil {
		panic(err)
	}
	defer subEndPointMessage.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
