package main

import (
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/geometry_msgs"
	"math"
	"os"
	"os/signal"
	"time"
)

type Point struct {
	Latitude  float64
	Longitude float64
}

var (
	IsStopped  bool
	IsEndPoint bool
	StartTime  int64
	count      int   = 0
	threshold  int64 = 3
	//定义园区长弯道坐标点
	EndPoint = Point{0, 0}
	point3   = Point{39.73004426154644, 116.49248639463602}

	pointlist  = []Point{point3}
	pointlist2 = []Point{EndPoint}
)

// 计算两点之间的距离（米）
func distance(point1, point2 Point) float64 {
	// 经纬度转弧度
	lat1 := point1.Latitude * math.Pi / 180
	lon1 := point1.Longitude * math.Pi / 180
	lat2 := point2.Latitude * math.Pi / 180
	lon2 := point2.Longitude * math.Pi / 180

	// 计算距离
	dlon := lon2 - lon1
	dlat := lat2 - lat1
	a := math.Sin(dlat/2)*math.Sin(dlat/2) + math.Sin(dlon/2)*math.Sin(dlon/2)*math.Cos(lat1)*math.Cos(lat2)
	c := 2 * math.Atan2(math.Sqrt(a), math.Sqrt(1-a))
	d := 6371000 * c

	return d
}
func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {

	if count%10 == 0 {
		//fmt.Println(threshold)
		enterflag := IfEnter(pointlist, 30, data.Latitude, data.Longitude)
		if enterflag {
			//fmt.Println("enterflag")
			IsEndPoint = IfEnter(pointlist2, 5.0, data.Latitude, data.Longitude)
			if !IsEndPoint {
				//fmt.Println("!IsEndPoint")
				AbsSpeed := math.Sqrt(math.Pow(data.VelocityX, 2) + math.Pow(data.VelocityY, 2))
				if AbsSpeed < 0.5 {
					// 如果之前没有记录开始时间，记录当前时间
					if StartTime == 0 {
						StartTime = time.Now().Unix()
					}

					// 判断是否持续超过一分钟
					if time.Now().Unix()-StartTime > threshold {
						if !IsStopped {
							event_label := "AbnormalStopOnCurve"
							fmt.Println(event_label)
							IsStopped = true
						}
					}
				} else {
					// 如果速度大于 0.1，重置开始时间和停止标志
					StartTime = 0
					IsStopped = false
				}
			}

		}
		count = 0
	}
	count++

}
func IfEnter(pointlist []Point, radius float64, x, y float64) bool {
	// 判断是否进入点列表中的区域
	point1 := Point{Latitude: x, Longitude: y}
	for _, point := range pointlist {
		d := distance(point1, point)
		//fmt.Println(d)
		if d <= radius {
			return true
		}
	}
	return false
}

func CallbackEndPointMessage(data *geometry_msgs.Point) {
	EndPoint.Longitude = data.X
	EndPoint.Latitude = data.Y

}
func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subEndPointMessage, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "/end_point_message",
		Callback: CallbackEndPointMessage,
	})
	if err != nil {
		panic(err)
	}
	defer subEndPointMessage.Close()
	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
