package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
	"time"
)

var (
	Framenum    int64 = 0
	param       entity.PjisuvParam
	ObjectSlice = make(map[uint32][][]float32)
)

func main() {
	ticker := time.NewTicker(5 * time.Second)
	defer ticker.Stop()
	go listener()
	for {
		select {
		case <-ticker.C:
			FinalCallback()
		}
	}
}

func isCuttingOut(ObjectList [][]float32) bool {
	for i := 0; i < len(ObjectList[0]); i++ {
		xi := ObjectList[0][i]
		yi := math.Abs(float64(ObjectList[1][i]))
		Type := ObjectList[4][0]
		if xi >= 0 && yi <= 0.7 && math.Abs(param.AngularVelocityZOfCicvLocation) <= 0.6 && (Type != 100.0) { //简化条件
			//if xi >= 0 && yi <= 0.7 && math.Abs(param.AngularVelocityZOfCicvLocation) <= 0.6 && (Type == 2.0 || Type == 3.0) {//理论条件
			for j := 0; j < len(ObjectList[0])-i-1; j++ {
				xj := ObjectList[0][j]
				yj := math.Abs(float64(ObjectList[1][j]))

				if xj >= 0 && yj >= 2.0 && math.Abs(param.AngularVelocityZOfCicvLocation) <= 0.6 {
					return true
				}
			}
		}
	}
	return false
}
func FinalCallback() {

	for _, objValue := range ObjectSlice {
		//fmt.Println(objValue)
		//fmt.Println("------------------------------------------------")

		if len(objValue[0]) <= 10 || !isCuttingOut(objValue) {
			continue
		}
		event_lable := "CCRH"
		fmt.Println(event_lable)
		ObjectSlice = make(map[uint32][][]float32)
	}

}

func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	param.YawOfCicvLocation = data.Yaw
	param.AngularVelocityZOfCicvLocation = data.AngularVelocityZ
}
func CallbackTpperception(data *pjisuv_msgs.PerceptionObjects) {
	Framenum += 1
	for _, obj := range data.Objs {
		if math.Abs(float64(obj.X)) >= 80 || math.Abs(float64(obj.Y)) >= 30 {
			continue
		}
		if _, ok := ObjectSlice[obj.Id]; !ok {
			ObjectSlice[obj.Id] = [][]float32{{}, {}, {}, {}, {}}
		}
		//absspeed := math.Sqrt(math.Pow(float64(obj.Vxabs), 2)+math.Pow(float64(obj.Vyabs), 2)) * 3.6
		ObjectSlice[obj.Id][0] = append(ObjectSlice[obj.Id][0], obj.X)
		ObjectSlice[obj.Id][1] = append(ObjectSlice[obj.Id][1], obj.Y)
		diffh := (float64(obj.Heading - float32(param.YawOfCicvLocation)))
		if diffh < -180.0 {
			diffh = 360.0 + diffh
		} else if diffh > 180.0 {
			diffh = 360.0 - diffh
		} else {
			diffh = math.Abs(diffh)
		}
		ObjectSlice[obj.Id][2] = append(ObjectSlice[obj.Id][2], float32(diffh))
		ObjectSlice[obj.Id][3] = append(ObjectSlice[obj.Id][3], float32(Framenum))
		ObjectSlice[obj.Id][4] = append(ObjectSlice[obj.Id][4], float32(obj.Type))

		//ObjectSlice[obj.Id][3] = append(ObjectSlice[obj.Id][3], float32(absspeed))
		if len(ObjectSlice[obj.Id][0]) >= 200 {
			ObjectSlice[obj.Id][0] = ObjectSlice[obj.Id][0][1:]
			ObjectSlice[obj.Id][1] = ObjectSlice[obj.Id][1][1:]
			ObjectSlice[obj.Id][2] = ObjectSlice[obj.Id][2][1:]
			ObjectSlice[obj.Id][3] = ObjectSlice[obj.Id][3][1:]
			ObjectSlice[obj.Id][4] = ObjectSlice[obj.Id][4][1:]

		}

	}
}
func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subTpperception, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "tpperception",
		Callback: CallbackTpperception,
	})
	if err != nil {
		panic(err)
	}
	defer subTpperception.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
