package main

import (
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
)

type Point struct {
	X float64
	Y float64
}

var (
	count     int     = 0
	threshold float64 = 23.0 / 3.6
	//定义园区长弯道坐标点
	point3 = Point{456508.573, 4397918.59}

	pointlist = []Point{point3}
)

func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {

	if count%10 == 0 {
		//print(threshold)
		enterflag := IfEnter(pointlist, 30, data.PositionX, data.PositionY)
		if enterflag {
			absspeed := math.Sqrt(math.Pow(data.VelocityX, 2) + math.Pow(data.VelocityY, 2))
			if absspeed >= threshold {
				eventLabel := "CurveOverspeed"
				fmt.Println(eventLabel)
			}

		}
		count = 0
	}
	count++

}
func IfEnter(pointlist []Point, radius float64, x, y float64) bool {
	// 判断是否进入点列表中的区域
	point1 := Point{X: x, Y: y}
	for _, point := range pointlist {
		d := distance(point1, point)
		//fmt.Println(d)
		if d <= radius {
			return true
		}
	}
	return false
}

// 计算两点之间的距离（米）
func distance(point1, point2 Point) float64 {

	d := math.Sqrt((point2.X-point1.X)*(point2.X-point1.X) + (point2.Y-point1.Y)*(point2.Y-point1.Y))

	return d
}
func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
