package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
	"time"
)

var (
	Frame       float32 = 0.0
	Maxlenobj   int32   = 0
	param       entity.PjisuvParam
	ObjectSlice = make(map[uint32][][]float32)
)

func main() {
	ticker := time.NewTicker(2 * time.Second)
	defer ticker.Stop()
	go listener()
	for {
		select {
		case <-ticker.C:
			FinalCallback()
		}
	}
}

func isCuttingIn(ObjectList [][]float32) (bool, float32, float32, float64) {
	for i, objY := range ObjectList[1] {

		if math.Abs(float64(objY)) >= 1.8 && math.Abs(param.AngularVelocityZOfCicvLocation) <= 0.6 {
			//fmt.Println(objY)
			for j := 0; j < len(ObjectList[1])-i-1; j++ {
				objX := ObjectList[0][1+i+j]
				if math.Abs(float64(ObjectList[1][1+i+j])) <= 0.7 && math.Abs(param.AngularVelocityZOfCicvLocation) <= 0.6 && objX >= 1 {
					//fmt.Println(objX)
					return true, ObjectList[2][1+i+j], ObjectList[0][1+i+j], math.Abs(float64(ObjectList[1][1+i+j]))
				}
			}
		}
	}
	return false, 0.0, 0.0, 0.0
}
func SightBlock(cutobjx float32, cutobjy float64, objId uint32, cutinframe float32) bool {

	for Id, objValue := range ObjectSlice {
		if Id != objId {
			for i := 0; i < len(objValue[1]); i++ {
				if objValue[2][i] == cutinframe {
					//fmt.Println("yes")
					objx := objValue[0][i]
					objy := math.Abs(float64(objValue[1][i]))
					//fmt.Println(objx)
					//fmt.Println(objy)
					diffx := cutobjx - objx
					diffy := math.Abs(cutobjy - objy)
					if diffx >= 0 && diffx <= 6 && diffy <= 4.0 {
						return true
					}
				}

			}

		}
	}
	return false
}
func FinalCallback() {
	for objId, objValue := range ObjectSlice {
		//fmt.Println(objValue)
		//fmt.Println("------------------------------------------------")
		Maxlenobj = max(Maxlenobj, int32(len(objValue[0])))
		cutinflag, cutinframe, cutobjx, cutobjy := isCuttingIn(objValue)
		if cutinflag {
			//fmt.Println(cutinflag, cutinframe, cutobjx, cutobjy)
			if SightBlock(cutobjx, cutobjy, objId, cutinframe) {
				event_lable := "CutinWithSightBblock"
				fmt.Println(event_lable)
				//ObjectSlice = make(map[uint32][][]float32)
			}
		}

	}
	if Maxlenobj >= 250 {
		ObjectSlice = make(map[uint32][][]float32)
		Maxlenobj = 0
	}
}

func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {

	param.AngularVelocityZOfCicvLocation = data.AngularVelocityZ
}
func CallbackTpperception(data *pjisuv_msgs.PerceptionObjects) {

	for _, obj := range data.Objs {
		if math.Abs(float64(obj.X)) >= 50 || math.Abs(float64(obj.Y)) >= 20 {
			continue
		}
		if _, ok := ObjectSlice[obj.Id]; !ok {
			ObjectSlice[obj.Id] = [][]float32{{}, {}, {}}
		}
		ObjectSlice[obj.Id][0] = append(ObjectSlice[obj.Id][0], obj.X)
		ObjectSlice[obj.Id][1] = append(ObjectSlice[obj.Id][1], obj.Y)
		ObjectSlice[obj.Id][2] = append(ObjectSlice[obj.Id][2], Frame)
	}
	Frame++
}
func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subTpperception, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "tpperception",
		Callback: CallbackTpperception,
	})
	if err != nil {
		panic(err)
	}
	defer subTpperception.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
