package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
	"time"
)

type Point struct {
	Latitude  float64
	Longitude float64
}

var (
	count2            = 0
	Maxlenobj   int32 = 0
	param       entity.PjisuvParam
	ObjectSlice = make(map[uint32][][]float32)
	//定义园区Ｔ字路口的经纬度坐标值
	point3 = Point{39.73040966605621, 116.48995329696209}
	point4 = Point{39.73083727413453, 116.49079780188244}
	point5 = Point{39.72976753711939, 116.49043130389033}
	point6 = Point{39.73012466515933, 116.49128381717591}
	point7 = Point{39.729251498328246, 116.49077484625299}
	point8 = Point{39.72964529630643, 116.49164592200161}

	pointlist = []Point{point3, point4, point5, point6, point7, point8}
)

func IfEnter(pointlist []Point, radius float64, lat, lon float64) bool {
	// 判断是否进入点列表中的区域
	point1 := Point{Latitude: lat, Longitude: lon}
	for _, point := range pointlist {
		d := distance(point1, point)
		if d <= radius {
			return true
		}
	}
	return false
}

// 计算两点之间的距离（米）
func distance(point1, point2 Point) float64 {
	// 经纬度转弧度
	lat1 := point1.Latitude * math.Pi / 180
	lon1 := point1.Longitude * math.Pi / 180
	lat2 := point2.Latitude * math.Pi / 180
	lon2 := point2.Longitude * math.Pi / 180

	// 计算距离
	dlon := lon2 - lon1
	dlat := lat2 - lat1
	a := math.Sin(dlat/2)*math.Sin(dlat/2) + math.Sin(dlon/2)*math.Sin(dlon/2)*math.Cos(lat1)*math.Cos(lat2)
	c := 2 * math.Atan2(math.Sqrt(a), math.Sqrt(1-a))
	d := 6371000 * c

	return d
}
func main() {
	ticker := time.NewTicker(2 * time.Second)
	defer ticker.Stop()
	go listener()
	for {
		select {
		case <-ticker.C:
			FinalCallback()
		}
	}
}

func isWrongDirection(ObjectList [][]float32) bool {
	//fmt.Println("-------------------------------------------------------------------------------------------------------------")
	//fmt.Println(ObjectList)
	for i, heading := range ObjectList[2] {
		objXi := ObjectList[0][i]
		objYi := ObjectList[1][i]
		Anglei := math.Abs(float64(heading - float32(param.YawOfCicvLocation)))
		//fmt.Println("objXi=", objXi)
		if math.Abs(float64(objYi)) <= 1.5 && math.Abs(param.AngularVelocityZOfCicvLocation) <= 0.6 && objXi >= 15 && Anglei <= 200 && Anglei >= 160 {
			//fmt.Println(objY)
			for j := 0; j < len(ObjectList[1])-i-1; j++ {
				objXij := ObjectList[0][1+i+j]
				objYij := ObjectList[1][1+i+j]
				//fmt.Println("objXij=", objXij)
				if math.Abs(float64(objYij)) <= 1.5 && math.Abs(param.AngularVelocityZOfCicvLocation) <= 0.6 && objXij <= 10 {
					return true
				}
			}
		}
	}
	return false
}
func FinalCallback() {

	for _, objValue := range ObjectSlice {
		//fmt.Println(objValue)
		//fmt.Println("------------------------------------------------")
		Maxlenobj = max(Maxlenobj, int32(len(objValue[0])))
		if len(objValue[0]) <= 10 || !isWrongDirection(objValue) || param.EnterJunctionFlag {
			continue
		}
		event_lable := "FrontCarRetrogradeOnJunction"
		ObjectSlice = make(map[uint32][][]float32)
		fmt.Println(event_lable)
	}
	if Maxlenobj >= 250 {
		ObjectSlice = make(map[uint32][][]float32)
		Maxlenobj = 0
	}
}

func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	if count2%10 == 0 {
		param.AngularVelocityZOfCicvLocation = data.AngularVelocityZ
		param.YawOfCicvLocation = data.Yaw
		param.EnterJunctionFlag = IfEnter(pointlist, 16, data.Latitude, data.Longitude)
		//fmt.Println(param.EnterJunctionFlag)

	}
	count2++
}

func CallbackTpperception(data *pjisuv_msgs.PerceptionObjects) {
	for _, obj := range data.Objs {
		if obj.X <= 2 || math.Abs(float64(obj.Y)) >= 80 {
			continue
		}
		if _, ok := ObjectSlice[obj.Id]; !ok {
			ObjectSlice[obj.Id] = [][]float32{{}, {}, {}}
		}
		ObjectSlice[obj.Id][0] = append(ObjectSlice[obj.Id][0], obj.X)
		ObjectSlice[obj.Id][1] = append(ObjectSlice[obj.Id][1], obj.Y)
		ObjectSlice[obj.Id][2] = append(ObjectSlice[obj.Id][2], obj.Heading)
	}
}
func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subTpperception, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "tpperception",
		Callback: CallbackTpperception,
	})
	if err != nil {
		panic(err)
	}
	defer subTpperception.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
