package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
)

var (
	param entity.PjisuvParam
	//D1, D2, D3 = 3.813, 0.952, 1.086  真实数据
	D1, D2, D3 = 1.0, 0.4, 0.4 //缩小后数据
)

type Point struct {
	X, Y float64
}

func main() {

	go listener()
	select {}
	//time.Sleep(10000)
}
func getVehicleCorners(x0, y0, D1, D2, D3, h0 float64) []pjisuv_msgs.Point64 {
	h0Rad := h0 * math.Pi / 180 // 转换角度到弧度
	cosH0, sinH0 := math.Cos(h0Rad), math.Sin(h0Rad)

	// 车辆四个角点相对于后轴中心点的坐标（局部坐标系）
	cornersLocal := []Point{
		{D1, D3},   // 前左角
		{D1, -D3},  // 前右角
		{-D2, D3},  // 后左角
		{-D2, -D3}, // 后右角
	}

	// 旋转矩阵并计算全局坐标
	rotationMatrix := [2][2]float64{{cosH0, -sinH0}, {sinH0, cosH0}}
	cornersGlobal := make([]pjisuv_msgs.Point64, len(cornersLocal))
	for i, corner := range cornersLocal {
		cornersGlobal[i].X = corner.X*rotationMatrix[0][0] + corner.Y*rotationMatrix[1][0] + x0
		cornersGlobal[i].Y = corner.X*rotationMatrix[0][1] + corner.Y*rotationMatrix[1][1] + y0
	}
	return cornersGlobal
}

func ccw(A, B, C pjisuv_msgs.Point64) bool {
	return (C.Y-A.Y)*(B.X-A.X) > (C.X-A.X)*(B.Y-A.Y)
}

func lineIntersect(A, B, C, D pjisuv_msgs.Point64) bool {
	return ccw(A, C, D) != ccw(B, C, D) && ccw(A, B, C) != ccw(A, B, D)
}
func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	param.PositionXOfCicvLocation = data.PositionX
	param.PositionYOfCicvLocation = data.PositionY
	param.LatitudeOfCicvLocation = data.Latitude
	param.LongitudeOfCicvLocation = data.Longitude
	param.YawOfCicvLocation = data.Yaw
}
func polygonLineIntersect(polygon []pjisuv_msgs.Point64, A, B pjisuv_msgs.Point64) bool {
	for i := 0; i < len(polygon)-1; i++ {
		C := polygon[i]
		D := polygon[(i+1)%len(polygon)]
		if lineIntersect(A, B, C, D) {
			return true
		}
	}

	return false
}
func CallbackMapPolygon(data *pjisuv_msgs.PolygonStamped) {
	defer func() {
		if r := recover(); r != nil {
			fmt.Println("Recovered from panic:", r)
		}
	}()
	Points := data.Polygon.Points
	corners := getVehicleCorners(param.PositionXOfCicvLocation, param.PositionYOfCicvLocation, D1, D2, D3, param.YawOfCicvLocation)

	for i := 0; i < len(Points)-1; i++ {
		A := Points[i]
		B := Points[i+1]
		if polygonLineIntersect(corners, A, B) {
			event_lable := "out_of_lane"
			fmt.Printf("PositionX= %f\n", param.PositionXOfCicvLocation-457000.0)
			fmt.Printf("PositionY= %f\n", param.PositionYOfCicvLocation-4400000.0)
			//fmt.Printf("Latitude= %f\n", param.LatitudeOfCicvLocation)
			//fmt.Printf("Longitude= %f\n", param.LongitudeOfCicvLocation)
			fmt.Printf("Yow= %f\n", param.YawOfCicvLocation)
			for i, corner := range corners {
				fmt.Printf("corners %d ,X=%f ,Y=%f\n", i, corner.X-457000.0, corner.Y-4400000.0)
			}
			fmt.Printf(event_lable)
			fmt.Println()

		}
	}

}
func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subMapPolygon, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "/map_polygon",
		Callback: CallbackMapPolygon,
	})
	if err != nil {
		panic(err)
	}
	defer subMapPolygon.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
