package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
	"time"
)

type Point struct {
	Latitude  float64
	Longitude float64
}

var (
	Maxlenobj   int32 = 0
	param       entity.PjisuvParam
	ObjectSlice = make(map[uint32][][]float32)
	pointcurve  = Point{39.73004426154644, 116.49248639463602}
	pointlist1  = []Point{pointcurve}
)

func main() {
	ticker := time.NewTicker(2 * time.Second)
	defer ticker.Stop()
	go listener()
	for {
		select {
		case <-ticker.C:
			FinalCallback()
		}
	}
}

func IsOvertaken(ObjectList [][]float32) bool {
	if param.AbsSpeed > 0.2 {
		//fmt.Println("yes")
		//fmt.Println(ObjectList)
		for i, objX := range ObjectList[0] {

			if math.Abs(float64(ObjectList[1][i])) <= 8 && objX >= 2.5 {
				for j := 0; j < len(ObjectList[0])-i-1; j++ {
					if math.Abs(float64(ObjectList[1][1+i+j])) <= 8 && ObjectList[0][1+i+j] <= -1.5 {
						return true
					}
				}
			}
		}
	}
	return false
}
func FinalCallback() {
	if IfEnter(pointlist1, 50, param.LatitudeOfCicvLocation, param.LongitudeOfCicvLocation) {
		//fmt.Println("here")
		for _, objValue := range ObjectSlice {
			//fmt.Println(ObjectSlice)
			//fmt.Println(objValue)
			//fmt.Println("------------------------------------------------")
			Maxlenobj = max(Maxlenobj, int32(len(objValue[0])))
			if len(objValue[0]) <= 5 || !IsOvertaken(objValue) {
				continue
			}
			event_lable := "OvertakingInCurve"
			fmt.Println(event_lable)
			ObjectSlice = make(map[uint32][][]float32)
		}
	}
	//fmt.Println(Maxlenobj)
	if Maxlenobj >= 100 {
		ObjectSlice = make(map[uint32][][]float32)
		Maxlenobj = 0
	}
}
func IfEnter(pointlist []Point, radius float64, lat, lon float64) bool {
	// 判断是否进入点列表中的区域
	point1 := Point{Latitude: lat, Longitude: lon}
	for _, point := range pointlist {
		d := distance(point1, point)
		if d <= radius {
			return true
		}
	}
	return false
}
func max(a, b int32) int32 {
	if a > b {
		return a
	}
	return b
}

// 计算两点之间的距离（米）
func distance(point1, point2 Point) float64 {
	// 经纬度转弧度
	lat1 := point1.Latitude * math.Pi / 180
	lon1 := point1.Longitude * math.Pi / 180
	lat2 := point2.Latitude * math.Pi / 180
	lon2 := point2.Longitude * math.Pi / 180

	// 计算距离
	dlon := lon2 - lon1
	dlat := lat2 - lat1
	a := math.Sin(dlat/2)*math.Sin(dlat/2) + math.Sin(dlon/2)*math.Sin(dlon/2)*math.Cos(lat1)*math.Cos(lat2)
	c := 2 * math.Atan2(math.Sqrt(a), math.Sqrt(1-a))
	d := 6371000 * c

	return d
}
func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	param.AbsSpeed = math.Sqrt(math.Pow(data.VelocityX, 2) + math.Pow(data.VelocityY, 2))
	param.LatitudeOfCicvLocation = data.Latitude
	param.LongitudeOfCicvLocation = data.Longitude

}
func CallbackTpperception(data *pjisuv_msgs.PerceptionObjects) {
	for _, obj := range data.Objs {
		if math.Abs(float64(obj.X)) >= 50 || math.Abs(float64(obj.Y)) >= 60 {
			continue
		}
		if _, ok := ObjectSlice[obj.Id]; !ok {
			ObjectSlice[obj.Id] = [][]float32{{}, {}}
		}
		ObjectSlice[obj.Id][0] = append(ObjectSlice[obj.Id][0], obj.X)
		ObjectSlice[obj.Id][1] = append(ObjectSlice[obj.Id][1], obj.Y)
		//ObjectSlice[obj.Id][2] = append(ObjectSlice[obj.Id][2], obj.Vxrel)
	}
}
func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subTpperception, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "tpperception",
		Callback: CallbackTpperception,
	})
	if err != nil {
		panic(err)
	}
	defer subTpperception.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
