package main

import (
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
)

var (
	threshold float64 = 15 / 3.6
	count1    int     = 0
)

func calculateDirectionAngle(speedX, speedY float64) float64 {
	// 使用反正切函数计算方向角
	angle := math.Atan2(speedY, speedX)

	// 将角度转换为度数
	angleDegree := angle * (180 / math.Pi)

	// 将角度限定在 0~360 范围内
	if angleDegree < 0 {
		angleDegree += 360
	} else if angleDegree > 360 {
		angleDegree -= 360
	}

	return angleDegree
}

func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	if count1%10 == 0 {

		directionAngle := calculateDirectionAngle(data.VelocityX, data.VelocityY)
		AbsSpeed := math.Sqrt(math.Pow(data.VelocityX, 2) + math.Pow(data.VelocityY, 2))
		diffAngle := math.Abs(float64(directionAngle - data.Yaw))
		//fmt.Println(diffAngle)
		if AbsSpeed >= threshold && diffAngle >= 160 && diffAngle <= 200 {
			eventLabel := "ReverseAndOverspeed"
			fmt.Println(eventLabel)
			count1 = 1
		}
	}
	count1++
}

func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
