package main

import (
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
)

type Point struct {
	Latitude  float64
	Longitude float64
	Id        string
}

var (
	count1     int = 0
	DicofLight     = make(map[string]int8)
	//定义园区４个信号灯的坐标
	point2 = Point{39.72998222922185, 116.49307473274646, "25174113"}

	pointlightlist = []Point{point2}
)

func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	if count1%10 == 0 {

		enterlightid := IfEnter(pointlightlist, 8.0, data.Latitude, data.Longitude)
		//fmt.Println(enterlightid)
		if enterlightid != "0" {
			if DicofLight[enterlightid] == 0 {
				eventLabel := "RunningRedLights"
				fmt.Println(eventLabel)
			}

		}
	}
	count1++
}
func Callbacktftrafficlight(data *pjisuv_msgs.TrafficLightDetection) {
	//fmt.Println("yes")
	for _, light := range data.TrafficLight {
		//fmt.Println(light.Id)
		//fmt.Println(light.Color)

		/*
			if _, ok := DicofLight[light.Id]; !ok {
				DicofLight[light.Id] = 9
			}

		*/
		DicofLight[light.Id] = light.Color
	}
	//fmt.Println(DicofLight)

}
func IfEnter(pointlist []Point, radius float64, lat, lon float64) string {
	// 判断是否进入点列表中的区域
	point1 := Point{Latitude: lat, Longitude: lon}
	for _, point := range pointlist {
		d := distance(point1, point)
		if d <= radius {
			return point.Id
		}
	}
	return "0"
}

// 计算两点之间的距离（米）
func distance(point1, point2 Point) float64 {
	// 经纬度转弧度
	lat1 := point1.Latitude * math.Pi / 180
	lon1 := point1.Longitude * math.Pi / 180
	lat2 := point2.Latitude * math.Pi / 180
	lon2 := point2.Longitude * math.Pi / 180

	// 计算距离
	dlon := lon2 - lon1
	dlat := lat2 - lat1
	a := math.Sin(dlat/2)*math.Sin(dlat/2) + math.Sin(dlon/2)*math.Sin(dlon/2)*math.Cos(lat1)*math.Cos(lat2)
	c := 2 * math.Atan2(math.Sqrt(a), math.Sqrt(1-a))
	d := 6371000 * c

	return d
}
func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subtftrafficlight, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/tftrafficlight",
		Callback: Callbacktftrafficlight,
	})
	if err != nil {
		panic(err)
	}
	defer subtftrafficlight.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
