package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
	"sort"
	"time"
)

func rank(data []float64) []float64 {
	n := len(data)
	ranked := make([]float64, n)
	for i := range ranked {
		ranked[i] = float64(i + 1)
	}
	sort.Float64s(data)

	for i := 0; i < n; {
		j := i
		for j < n && data[i] == data[j] {
			j++
		}
		rank := (float64(i+1) + float64(j)) / 2
		for k := i; k < j; k++ {
			ranked[k] = rank
		}
		i = j
	}

	return ranked
}

func mannWhitneyU(sampleA, sampleB []float64) (float64, float64) {
	n1 := float64(len(sampleA))
	n2 := float64(len(sampleB))

	all := append(sampleA, sampleB...)
	ranks := rank(all)

	rankSumA := 0.0
	for i := 0; i < len(sampleA); i++ {
		rankSumA += ranks[i]
	}

	u1 := rankSumA - n1*(n1+1)/2
	u2 := n1*n2 - u1

	return u1, u2
}

func calculatePValue(u, n1, n2 float64) float64 {
	meanU := n1 * n2 / 2
	sdU := math.Sqrt(n1 * n2 * (n1 + n2 + 1) / 12)
	z := (u - meanU) / sdU
	pValue := 2 * (1 - normalCDF(math.Abs(z)))
	return pValue
}

func normalCDF(x float64) float64 {
	return 0.5 * (1 + math.Erf(x/math.Sqrt2))
}

var (
	countSideBySide = 0
	param           entity.PjisuvParam
)

func main() {
	ticker := time.NewTicker(3 * time.Second)
	defer ticker.Stop()
	go listener()
	for {
		select {
		case <-ticker.C:
			FinalCallback()
		}
	}
}
func FinalCallback() {
	if countSideBySide >= 60 {
		event_lable := "FrontCarSideBySide"
		fmt.Println(event_lable)
	}
	countSideBySide = 0
}
func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	param.AbsSpeed = math.Sqrt(math.Pow(data.VelocityX, 2) + math.Pow(data.VelocityY, 2))

}
func CallbackTpperception(data *pjisuv_msgs.PerceptionObjects) {
lable1:
	for _, obj := range data.Objs {
		//if !(obj.Type != 2 || obj.Type != 3 || obj.Type != 4 || !(obj.X >= 3 && obj.X <= 17) || math.Abs(float64(obj.Y)) > 10)
		if !(!(obj.X >= 3 && obj.X <= 17) || math.Abs(float64(obj.Y)) > 10) {
			//fmt.Println("here")
			obj1ID := obj.Id
			obj1x := obj.X
			obj1y := obj.Y
			//fmt.Println(obj1ID, obj1x, obj1y)
			for _, obj1 := range data.Objs {
				//if !(obj1.Type != 2 || obj1.Type != 3 || obj1.Type != 4 || !(obj1.X >= 3 && obj1.X <= 17) || math.Abs(float64(obj1.Y)) > 10)
				if !(!(obj1.X >= 3 && obj1.X <= 17) || math.Abs(float64(obj1.Y)) > 10) {
					obj2ID := obj1.Id
					obj2x := obj1.X
					obj2y := obj1.Y
					if obj2ID != obj1ID && math.Abs(float64(obj2x-obj1x)) <= 2.2 && math.Abs(float64(obj2y-obj1y)) >= 2 && math.Abs(float64(obj2y-obj1y)) <= 4.0 && param.AbsSpeed > 2 {
						countSideBySide++
						break lable1
					}
				}

			}

		}

	}
}
func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subTpperception, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "tpperception",
		Callback: CallbackTpperception,
	})
	if err != nil {
		panic(err)
	}
	defer subTpperception.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
