package main

import (
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
)

type Point struct {
	Latitude  float64
	Longitude float64
}

var (
	count     int     = 0
	threshold float64 = 19.0 / 3.6
	//定义园减速带经纬度
	point3    = Point{39.73002669559404, 116.49020088105833}
	point4    = Point{39.728688903362254, 116.4896802037327}
	point5    = Point{39.728406077854494, 116.48901074348075}
	point6    = Point{39.7287051534828, 116.48822784823321}
	point7    = Point{39.72925799042968, 116.49074368869654}
	pointlist = []Point{point3, point4, point5, point6, point7}
)

func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {

	if count%10 == 0 {
		//print(threshold)
		enterflag := IfEnter(pointlist, 7, data.Latitude, data.Longitude)
		if enterflag {
			//fmt.Println("yes")
			absspeed := math.Sqrt(math.Pow(data.VelocityX, 2) + math.Pow(data.VelocityY, 2))
			if absspeed >= threshold {
				eventLabel := "SpeedBumpOverspeed"
				fmt.Println(eventLabel)
			}

		}
		count = 0
	}
	count++

}
func IfEnter(pointlist []Point, radius float64, x, y float64) bool {
	// 判断是否进入点列表中的区域
	point1 := Point{Latitude: x, Longitude: y}
	for _, point := range pointlist {
		d := distance(point1, point)
		//fmt.Println(d)
		if d <= radius {
			return true
		}
	}
	return false
}

func distance(point1, point2 Point) float64 {
	// 经纬度转弧度
	lat1 := point1.Latitude * math.Pi / 180
	lon1 := point1.Longitude * math.Pi / 180
	lat2 := point2.Latitude * math.Pi / 180
	lon2 := point2.Longitude * math.Pi / 180

	// 计算距离
	dlon := lon2 - lon1
	dlat := lat2 - lat1
	a := math.Sin(dlat/2)*math.Sin(dlat/2) + math.Sin(dlon/2)*math.Sin(dlon/2)*math.Cos(lat1)*math.Cos(lat2)
	c := 2 * math.Atan2(math.Sqrt(a), math.Sqrt(1-a))
	d := 6371000 * c

	return d
}
func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
