package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"math"
	"os"
	"os/signal"
)

type Point struct {
	Latitude  float64
	Longitude float64
}

var (
	param       entity.PjisuvParam
	count1      int64
	ObjectSlice = make(map[uint32][][]float32)
)

func IfObstaclesNearby() bool {
	for _, obj := range ObjectSlice {
		if obj[0][len(obj[0])-1] >= -15 && obj[0][len(obj[0])-1] <= -1 && (math.Abs(float64(obj[1][len(obj[1])-1]))) <= 6 {
			return true

		}
	}
	return false
}
func calculateDirectionAngle(speedX, speedY float64) float64 {
	// 使用反正切函数计算方向角
	angle := math.Atan2(speedY, speedX)

	// 将角度转换为度数
	angleDegree := angle * (180 / math.Pi)

	// 将角度限定在 0~360 范围内
	if angleDegree < 0 {
		angleDegree += 360
	} else if angleDegree > 360 {
		angleDegree -= 360
	}

	return angleDegree
}

func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	//print(param.AutomodeOfPjVehicleFdbPub)

	if count1%10 == 0 {

		directionAngle := calculateDirectionAngle(data.VelocityX, data.VelocityY)
		AbsSpeed := math.Sqrt(math.Pow(data.VelocityX, 2) + math.Pow(data.VelocityY, 2))
		diffAngle := math.Abs(float64(directionAngle - data.Yaw))
		flag := IfObstaclesNearby()
		//fmt.Println(ObjectSlice)
		if diffAngle >= 160 && diffAngle <= 200 && flag && AbsSpeed >= 1 {
			event_label := "TargetCarBehindWhenReversing"
			fmt.Println(event_label)
		}

		count1 = 1
	} else {
		count1++
	}

}

func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}
func CallbackTpperception(data *pjisuv_msgs.PerceptionObjects) {
	for _, obj := range data.Objs {
		if obj.X <= -2 || math.Abs(float64(obj.Y)) >= 90 {
			continue
		}
		if _, ok := ObjectSlice[obj.Id]; !ok {
			ObjectSlice[obj.Id] = [][]float32{{}, {}}
		}
		ObjectSlice[obj.Id][0] = append(ObjectSlice[obj.Id][0], obj.X)
		ObjectSlice[obj.Id][1] = append(ObjectSlice[obj.Id][1], obj.Y)
	}
}
func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// create a subscriber
	subTpperception, err := goroslib.NewSubscriber(goroslib.SubscriberConf{

		Node:     n,
		Topic:    "/tpperception",
		Callback: CallbackTpperception,
	})
	if err != nil {
		panic(err)
	}
	defer subTpperception.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
