package main

import (
	"awesomeProject/entity"
	"awesomeProject/pjisuv_msgs"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"os"
	"os/signal"
	"time"
)

type Point struct {
	x, y float64
}

var (
	param    entity.PjisuvParam
	vertices = []Point{
		{x: 456128.413, y: 4397847.78},
		{x: 456288.257, y: 4397953.51},
		{x: 456359.022, y: 4397822.84},
		{x: 456191.065, y: 4397733.3},
	}
)

func isPointInPolygon(p Point, vertices []Point) bool {
	intersections := 0
	for i := 0; i < len(vertices); i++ {
		j := (i + 1) % len(vertices)
		if rayIntersectsSegment(p, vertices[i], vertices[j]) {
			intersections++
		}
	}
	return intersections%2 == 1
}

func rayIntersectsSegment(p, p1, p2 Point) bool {
	if p1.y > p2.y {
		p1, p2 = p2, p1
	}
	if p.y == p1.y || p.y == p2.y {
		p.y += 0.0001
	}

	if p.y < p1.y || p.y > p2.y {
		return false
	}

	if p.x > max(p1.x, p2.x) {
		return false
	}

	if p.x < min(p1.x, p2.x) {
		return true
	}

	blueSlope := (p.y-p1.y)*(p2.x-p1.x)/(p2.y-p1.y) + p1.x
	return p.x < blueSlope
}

func min(a, b float64) float64 {
	if a < b {
		return a
	}
	return b
}

func max(a, b float64) float64 {
	if a > b {
		return a
	}
	return b
}
func main() {
	ticker := time.NewTicker(1 * time.Second)
	defer ticker.Stop()
	go listener()
	for {
		select {
		case <-ticker.C:
			FinalCallback()
		}
	}
}
func FinalCallback() {

	p := Point{x: param.PositionXOfCicvLocation, y: param.PositionYOfCicvLocation}
	if isPointInPolygon(p, vertices) {
		fmt.Println("EgoCar inside.")
	} else {
		fmt.Println("EgoCar Outside.")
	}
}
func CallbackCicvLocation(data *pjisuv_msgs.PerceptionLocalization) {
	param.PositionXOfCicvLocation = data.PositionX
	param.PositionYOfCicvLocation = data.PositionY
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/cicv_location",
		Callback: CallbackCicvLocation,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
