package main

import (
	"awesomeProject/entity"
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/nav_msgs"
	"math"
	"os"
	"os/signal"
	"time"
)

var (
	param     entity.PjisuvParam
	StartTime int64
	IsStopped bool
)

func CallbackWheelOdom(data *nav_msgs.Odometry) {
	velocityX := data.Twist.Twist.Linear.X
	velocityY := data.Twist.Twist.Linear.Y
	absV := math.Pow(math.Pow(velocityX, 2)+math.Pow(velocityY, 2), 0.5)
	if absV < 0.3 {
		// 如果之前没有记录开始时间，记录当前时间
		if StartTime == 0 {
			StartTime = time.Now().Unix()
		}

		// 判断是否持续超过一分钟
		if time.Now().Unix()-StartTime > 10 {
			if !IsStopped {
				event_label := "AuLongStop"
				fmt.Println(event_label)
				IsStopped = true
			}
		}
	} else {
		// 如果速度大于 0.1，重置开始时间和停止标志
		StartTime = 0
		//is_stopped = false
	}

}

func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subCicvLocation, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/wheel_odom",
		Callback: CallbackWheelOdom,
	})
	if err != nil {
		panic(err)
	}
	defer subCicvLocation.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
