package main

import (
	"fmt"
	"github.com/bluenviron/goroslib/v2"
	"github.com/bluenviron/goroslib/v2/pkg/msgs/geometry_msgs"
	"math"
	"os"
	"os/signal"
)

type Point struct {
	X float64
	Y float64
}

var (
	count2 int = 0
	point3     = Point{-29.5885, 108.087}
	point4     = Point{209.923, 210.394}
	point5     = Point{16.441, -37.0252}
	point6     = Point{-53.9421, -74.9124}
	point7     = Point{232.502, 172.084}
	//定义园区部分井盖坐标值

	pointlist = []Point{point3, point4, point5, point6, point7}
)

func CallbackRobotPose(data *geometry_msgs.PoseStamped) {
	if count2%10 == 0 {

		enterflag := IfEnter(pointlist, 12, data.Pose.Position.X, data.Pose.Position.Y)
		if enterflag {
			eventLabel := "PassManholeCover"
			fmt.Println(eventLabel)
		}
	}
	count2++
}
func IfEnter(pointlist []Point, radius float64, x, y float64) bool {
	// 判断是否进入点列表中的区域
	point1 := Point{X: x, Y: y}
	for _, point := range pointlist {
		d := distance(point1, point)
		if d <= radius {
			return true
		}
	}
	return false
}

// 计算两点之间的距离（米）
func distance(point1, point2 Point) float64 {
	d := math.Sqrt((point2.X-point1.X)*(point2.X-point1.X) + (point2.Y-point1.Y)*(point2.Y-point1.Y))

	return d
}
func main() {
	go listener()
	select {}
	//time.Sleep(10000)
}

func listener() {
	// create a node and connect to the master
	n, err := goroslib.NewNode(goroslib.NodeConf{
		Name:          "goroslib_sub",
		MasterAddress: "127.0.0.1:11311",
	})
	if err != nil {
		panic(err)
	}
	defer n.Close()

	// create a subscriber
	subRobotPose, err := goroslib.NewSubscriber(goroslib.SubscriberConf{
		Node:     n,
		Topic:    "/robot_pose",
		Callback: CallbackRobotPose,
	})
	if err != nil {
		panic(err)
	}
	defer subRobotPose.Close()

	// wait for CTRL-C
	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)
	<-c
}
